from datetime import datetime
import calendar

from textual.app import App, ComposeResult
from textual.containers import Container, Horizontal, Vertical
from textual.widgets import Button, Footer, Header, Static, Label, Placeholder, Tree
from textual.widget import Widget
from textual.messages import Message
from textual import log
from textual.reactive import var, reactive
from textual.pilot import Pilot
from rich.text import Text
from rich.table import Table
from rich.style import Style
from rich import box
from rich.table import Column

import girok.api.category as category_api
import girok.utils.calendar as calendar_utils
import girok.utils.display as display_utils

from girok.calendar_cli.calendar_app import CalendarApp
from girok.calendar_cli.calendar_container import CalendarContainer, Calendar
from girok.calendar_cli.sidebar import CategoryTree, SidebarContainer, TagTree
import girok.constants as constants


class Entry(App):
    CSS_PATH = "./calendar_main.css"
    current_focused = "CategoryTree"
    is_pop_up = False
    BINDINGS = [
        ("q", "quit", "Quit Nuro"),
        ("u", "show_previous_month", "Show prev month"),
        ("i", "show_next_month", "Show next month"),
        ("y", "show_current_month", "Show current month"),
        ("ctrl+l", "focus_on_calendar", "Move to calendar"),
        ("ctrl+g", "focus_on_sidebar", "Move to sidebar"),
        ("ctrl+j", "move_down_to_tag_tree", "Move down to tag tree"),
        ("ctrl+k", "move_up_to_category_tree", "Move up to category tree"),
        ("o", 'close_pop_up', "Close pop up box"),
        ("f", "toggle_files", "Toggle Files")
    ]
    show_sidebar = reactive(True)
    pilot = None
    
    def on_mount(self):
        self.set_focus(self.query_one(CategoryTree))
        
    def compose(self):
        yield CalendarApp()
        # yield Footer()
        
    # Display pop-up box when selecting a cell
    def on_calendar_task_cell_selected(self, event: Calendar.TaskCellSelected):
        cell_tasks = event.cell_tasks
        year, month, day = event.year, event.month, event.day
        table = display_utils.build_task_table(cell_tasks)
        
        self.query_one(CalendarContainer).mount(
            Vertical(
                Static(Text(f"{day} {calendar.month_name[month]} {year}", style=Style(bold=True, color=constants.TABLE_HEADER_DATE_COLOR)), classes="task-pop-up-header"),
                Container(Static(table, classes="task-pop-up-table"), classes="task-pop-up-table-container"),
                classes='task-pop-up-container',
            )
        )
        self.is_pop_up = True
        
    def action_quit(self):
        self.exit()
       
    def action_show_next_month(self):
        if self.is_pop_up:
            return
        calendar_container = self.query_one(CalendarContainer)
        calendar_container.update_month_by_offset(1)
         
    def action_show_previous_month(self):
        if self.is_pop_up:
            return
        calendar_container = self.query_one(CalendarContainer)
        calendar_container.update_month_by_offset(-1)
        
    def action_show_current_month(self):
        if self.is_pop_up:
            return
        calendar_container = self.query_one(CalendarContainer)
        now = datetime.now()
        cur_year, cur_month = now.year, now.month
        calendar_container.update_year_and_month(cur_year, cur_month)
        
    def action_focus_on_calendar(self):
        if self.is_pop_up:
            return
        self.set_focus(self.query_one(Calendar))
        self.current_focused = "Calendar"
        
        cat_tree = self.query_one(CategoryTree)
        tag_tree = self.query_one(TagTree)
        calendar_utils.remove_left_arrow_tree(cat_tree.highlighted_node)
        calendar_utils.remove_left_arrow_tree(tag_tree.highlighted_node)
        calendar_utils.remove_highlight(cat_tree.highlighted_node)
        calendar_utils.remove_highlight(tag_tree.highlighted_node)
        
    def action_focus_on_sidebar(self):
        if self.is_pop_up:
            return
        self.set_focus(self.query_one(CategoryTree))
        self.current_focused = "CategoryTree"
        
        cal = self.query_one(Calendar)
        
        # self.pilot.hover(Calendar)
        
        if self.is_pop_up:
            return
        
        cat_tree = self.query_one(CategoryTree)
        calendar_utils.remove_left_arrow(cal.cur_focused_cell)
        calendar_utils.add_highlight(cat_tree.highlighted_node)
        
        ############# UNKNOWN ERROR - Temporary Fix ############
        ngbrs = [self.query_one("#cell0"), self.query_one("#cell1"), self.query_one("#cell2"), self.query_one("#cell3"), self.query_one("#cell4"), self.query_one("#cell5"), self.query_one("#cell6")]
        for ngbr_cell in ngbrs:
            temp = []
            while ngbr_cell.children:
                child = ngbr_cell.children[0]
                temp.append(child.render())
                child.remove()
                
            for child in temp:
                ngbr_cell.mount(Static(child))
        ######################################################
        
    def action_move_down_to_tag_tree(self):
        if self.is_pop_up:
            return
        if self.current_focused != "CategoryTree":
            return
        tag_tree = self.query_one(TagTree)
        self.set_focus(tag_tree)
        self.current_focused = "TagTree"
        category_tree = self.query_one(CategoryTree)
        calendar_utils.remove_highlight(category_tree.highlighted_node)
        calendar_utils.remove_left_arrow_tree(category_tree.highlighted_node)
        calendar_utils.add_left_arrow_tree(tag_tree.highlighted_node)
        calendar_utils.add_highlight(tag_tree.highlighted_node)
    
    def action_move_up_to_category_tree(self):
        if self.is_pop_up:
            return
        if self.current_focused != "TagTree":
            return
        category_tree = self.query_one(CategoryTree)
        self.set_focus(category_tree)
        self.current_focused = "CategoryTree"
        tag_tree = self.query_one(TagTree)
        calendar_utils.remove_highlight(tag_tree.highlighted_node)
        calendar_utils.remove_left_arrow_tree(tag_tree.highlighted_node)
        calendar_utils.add_left_arrow_tree(category_tree.highlighted_node)
        calendar_utils.add_highlight(category_tree.highlighted_node)
        
    def action_close_pop_up(self):
        if not self.is_pop_up:
            return
        self.query_one(".task-pop-up-container").remove()
        self.query_one(Calendar).is_pop_up = False
        self.is_pop_up = False
        
    def action_toggle_files(self):
        self.show_sidebar = not self.show_sidebar
        sidebar_container = self.query_one(SidebarContainer)
        if self.show_sidebar:
            sidebar_container.styles.display = "block"
        else:
            sidebar_container.styles.display = "none"
            
            
if __name__ == '__main__':
   app = Entry()
   app.run()


import requests
from girok.config import get_config
cfg = get_config()

base_url = cfg.base_url 

def register(email, password):
    resp = requests.post(base_url + "/register", json={
        "email": email,
        "password": password
    })
    return resp
    

def login(email, password):
    files = { 
        "username": (None, email),
        "password": (None, password)
    }
    
    resp = requests.post(base_url + "/login", files=files)
    return resp


def validate_access_token(access_token):
    options = {
        "headers": {
        "Authorization": "Bearer " + access_token,
        }
    }
    

    resp = requests.get(base_url + "/validate-access-token", headers=options['headers'])
    return resp


from pathlib import Path
import os
import os.path as osp

import typer
from rich import print
from rich.console import Console
from rich.table import Table

from girok.config import get_config
import girok.api.auth as auth_api
import girok.commands.auth as auth_command
import girok.commands.category as category_command
import girok.commands.task as task_command
import girok.commands.calendar as calendar_command
import girok.utils.general as general_utils
import girok.utils.auth as auth_utils

app = typer.Typer(rich_markup_mode='rich')
app.registered_commands.extend(auth_command.app.registered_commands)
app.registered_commands.extend(category_command.app.registered_commands)
app.registered_commands.extend(task_command.app.registered_commands)
app.registered_commands.extend(calendar_command.app.registered_commands)
cfg = get_config()
    
@app.command()
def test():
    print("test")


@app.callback()
def pre_command_callback(ctx: typer.Context):
    # Setting up app dir and config file if they don't exist
    general_utils.config_setup(cfg.app_dir, cfg.config_path)
    if ctx.invoked_subcommand in ['login', 'logout', 'register']:
        return
        
    # Check if JWT is stored in config file
    stored_access_token = auth_utils.get_access_token_from_json(cfg.config_path)
    if stored_access_token:
        resp = auth_api.validate_access_token(stored_access_token)
        if resp.status_code != 200: # invalid(or expired) JWT -> login again
            print("You're not logged in. Please login with [green]girok login[/green].")
            exit(0)
    else:
        print("You're not logged in. Please login with [green]girok login[/green].")
        exit(0)
        


from datetime import datetime
import calendar

from rich import print
from rich.console import Console
from rich.tree import Tree
from rich.align import Align
from rich import box
from rich.style import Style
from rich.table import Table, Column
from rich.text import Text
from rich.align import Align
import shutil

import girok.utils.task as task_utils
import girok.utils.calendar as calendar_utils
import girok.utils.general as general_utils
from girok.config import get_config
import girok.constants as constants

console = Console()
cfg = get_config()

def display_categories(cats_dict: dict, highlight_cat=None):
    tree = Tree("")
    for idx, cat in enumerate(cats_dict):
        display_category(
            cats_dict,
            cat,
            top_most=True,
            tree=tree,
            cumul_path=cat,
            highlight_cat=highlight_cat,
            color=cats_dict[cat]['color']
        )
    console.print(tree)
    
    
def display_category(
    cats_dict: dict,
    cat: str,
    top_most=True,
    tree: Tree=None,
    cumul_path="",
    highlight_cat=None,
    color=None,
):
    style = Style(
        color=constants.TABLE_HEADER_COLOR if highlight_cat != cumul_path else "#B9D66A",
        bold=True
    )
    # if not top_most:
    #     style = Style(color=constants.TABLE_TASK_NAME_COLOR, bold=True)
        
    # if top_most:
    circle = Text(constants.CIRCLE_EMOJI + " ", style=Style(color=constants.CIRCLE_COLOR[color]))
        
    dir_name = Text(f"{cat}", style=style)
    # if top_most:
    dir_name = Text.assemble(circle, dir_name)
        
    tree = tree.add(dir_name)
    for sub in cats_dict[cat]['subcategories'].keys():
        display_category(
            cats_dict[cat]['subcategories'],
            sub,
            top_most=False,
            tree=tree,
            cumul_path=cumul_path + "/" + sub,
            highlight_cat=highlight_cat,
            color=color
        )
        
    
def center_print(text, type: str = None, wrap: bool = False):
    if type == "title":
        style = Style(
            color=constants.DISPLAY_TERMINAL_COLOR_TITLE_TEXT,
            bgcolor=constants.DISPLAY_TERMINAL_COLOR_TITLE_BACKGROUND
        )
    elif type == "success":
        style = Style(
            color=constants.DISPLAY_TERMINAL_COLOR_SUCCESS_TEXT,
            bgcolor=constants.DISPLAY_TERMINAL_COLOR_SUCCESS_BACKGROUND
        )
    elif type == "error":
        style = Style(
            color=constants.DISPLAY_TERMINAL_COLOR_ERROR_TEXT,
            bgcolor=constants.DISPLAY_TERMINAL_COLOR_ERROR_BACKGROUND
        )
    elif type == "warning":
        style = Style(
            color=constants.DISPLAY_TERMINAL_COLOR_WARNING_TEXT,
            bgcolor=constants.DISPLAY_TERMINAL_COLOR_WARNING_BACKGROUND 
        )
    else:
        style = Style(
            color=constants.DISPLAY_TERMINAL_COLOR_TITLE_TEXT,
            bgcolor=constants.DISPLAY_TERMINAL_COLOR_TITLE_BACKGROUND
        )
    if wrap:
        width = shutil.get_terminal_size().columns // 2
    else:
        width = shutil.get_terminal_size().columns
        
    content = Text(text, style=style)
    console.print(Align.center(content, style=style, width=width), height=100)
    
    
def display_tasks_by_category(
    task_tree,
    color_dict,
    marked_task_id: int = None,
    marked_color: str = constants.TABLE_TASK_DELETED_COLOR
):
    # print(task_tree)
    task_ids_cache = {}
    tree = Tree("")
    for idx, cat_name in enumerate(task_tree):
        tree.add(display_category_with_tasks(
            cat_name=cat_name,
            task_tree=task_tree[cat_name],
            color_dict=color_dict,
            top_most=True,
            task_ids_cache=task_ids_cache,
            marked_task_id=marked_task_id,
            marked_color=marked_color
            ))
    general_utils.cache_task_ids(cfg=cfg, cache=task_ids_cache)
    return tree


def display_category_with_tasks(
    cat_name,
    task_tree: dict,
    color_dict,
    top_most,
    task_ids_cache,
    marked_task_id: int = None,
    marked_color: str = constants.TABLE_TASK_DELETED_COLOR,
):
    dir_name = Text(f"{cat_name}", style=Style(color=constants.TABLE_HEADER_COLOR, bold=True))
    dir_name = Text.assemble(
        Text(f"{constants.CIRCLE_EMOJI} ", style=Style(color=constants.CIRCLE_COLOR[color_dict[cat_name]])),
        dir_name
    )
        
    tree = Tree(dir_name)
    
    for sub_cat_name in task_tree['sub_categories']:
        sub_tree = display_category_with_tasks(
            cat_name=sub_cat_name,
            task_tree=task_tree['sub_categories'][sub_cat_name],
            color_dict=color_dict,
            top_most=False,
            task_ids_cache=task_ids_cache,
            marked_task_id=marked_task_id,
            marked_color=marked_color
        )
        tree.add(sub_tree)
        
    for task in task_tree['tasks']:
        # Cache task id
        if not(task['task_id'] == marked_task_id and marked_color == constants.TABLE_TASK_DELETED_COLOR):
            task_ids_cache[len(task_ids_cache) + 1] = task['task_id']
        
        deadline = datetime.strptime(task['deadline'], "%Y-%m-%dT%H:%M:%S")
        year, month, day = deadline.year, deadline.month, deadline.day
        h, m, s = str(deadline.time()).split(":")
        
        weekday_name = task_utils.get_weekday_name_from_date(year, month, day)
        
        h = int(h)
        is_time = task['is_time']
        afternoon = h >= 12
        if h > 12:
            h -= 12
            
        time = f" / {h}:{m} {'PM' if afternoon else 'AM'}" if is_time else ''
        remaining_days = task_utils.get_day_offset_between_two_dates(datetime.now(), deadline)
        day_offset_message = f"{remaining_days} days left" if remaining_days > 0 else f"{abs(remaining_days)} days passed"
        
        if remaining_days == 0:
            day_offset_message = "Due Today"
            
        if not(task['task_id'] == marked_task_id and marked_color == constants.TABLE_TASK_DELETED_COLOR):
            task_id = Text(f"[{len(task_ids_cache)}] ", style=Style(color="white" if task['task_id'] != marked_task_id else marked_color))
        else:
            task_id = ""
        task_name = Text(task['name'], style=Style(color=constants.TABLE_TASK_NAME_COLOR  if task['task_id'] != marked_task_id else marked_color, bold=True))
        task_remaining_days = Text(f" [{day_offset_message}]", style=Style(color='#F5F7F2' if task['task_id'] != marked_task_id else marked_color))
        task_date = Text(f" - {year}/{month}/{day} {weekday_name}", style=Style(color="white" if task['task_id'] != marked_task_id else marked_color))
        node_name = Text.assemble(task_id, task_name, task_remaining_days, task_date, style=Style(strike=True if task['task_id'] == marked_task_id and marked_color == constants.TABLE_TASK_DELETED_COLOR else False))
             
        tree.add(node_name)
        
    return tree        
            
            
def display_tasks_by_list(tasks: list, marked_task_id: int = None, color: str = constants.TABLE_TASK_DELETED_COLOR):
    table = build_task_table(tasks, marked_task_id, color)
    print(Align(table, align="center"))
    
    
def build_task_table(tasks: list, marked_task_id: int = None, color: str = constants.TABLE_TASK_DELETED_COLOR):
    tasks = sorted(tasks, key=lambda x: calendar_utils.get_date_obj_from_str_separated_by_T(x['deadline']))
    num_tasks = len(tasks)
    table = Table(
        Column(header="ID", justify="center", header_style=Style(color=constants.TABLE_HEADER_COLOR), style=Style(color=constants.TABLE_TASK_NAME_COLOR)),
        Column(header="Name", header_style=Style(color=constants.TABLE_HEADER_COLOR), style=Style(color=constants.TABLE_TASK_NAME_COLOR)),
        Column(header="Category", justify="center", header_style=Style(color=constants.TABLE_HEADER_COLOR), style=Style(color=constants.TABLE_TEXT_COLOR)),
        Column(header="Deadline", justify="center", header_style=Style(color=constants.TABLE_HEADER_COLOR), style=Style(color="green")),
        Column(header="Tag", justify="center", header_style=Style(color=constants.TABLE_HEADER_COLOR), style=Style(color=constants.TABLE_TEXT_COLOR)),
        Column(header="Priority", justify="center", header_style=Style(color=constants.TABLE_HEADER_COLOR), style=Style(color=constants.TABLE_TEXT_COLOR)),
        Column(header="Remaining", justify="center", header_style=Style(color=constants.TABLE_HEADER_COLOR), style=Style(color=constants.TABLE_TEXT_COLOR)),
        box=box.SIMPLE_HEAVY,
        show_lines=False if num_tasks > 15 else True,
        border_style=Style(color="#D7E1C9", bold=True)
    )
    
    task_ids_cache = {}
    for idx, task in enumerate(tasks):
        # Cache task id
        task_ids_cache[idx + 1] = task['task_id']
        
        date = calendar_utils.get_date_obj_from_str_separated_by_T(task['deadline'])
        deadline = f"{date.day} {calendar.month_name[date.month]} {date.year}"
        remaining_days = task_utils.get_day_offset_between_two_dates(datetime.now(), date)
        day_offset_message = f"{remaining_days} days left" if remaining_days > 0 else f"{abs(remaining_days)} days passed"
        if remaining_days == 0:
            day_offset_message = "Due Today"
        circle = Text(constants.CIRCLE_EMOJI + " ", style=Style(color=constants.CIRCLE_COLOR[task['color']]))
        task_name = Text.assemble(circle, task['name'], style=Style(color=color if marked_task_id == task['task_id'] else constants.TABLE_TASK_NAME_COLOR, bold=True))
        
        table.add_row(
            Text(str(idx + 1), style=Style(color=color if marked_task_id == task['task_id'] else constants.TABLE_TEXT_COLOR)),
            task_name,
            Text(task['task_category_full_path'] if task['task_category_full_path'] else "None", style=Style(color=color if marked_task_id == task['task_id'] else constants.TABLE_TEXT_COLOR)),
            Text(deadline, style=Style(color=color if marked_task_id == task['task_id'] else constants.TABLE_DEADLINE_COLOR)),
            Text(task['tag'] if task['tag'] else '-', style=Style(color=color if marked_task_id == task['task_id'] else constants.TABLE_TEXT_COLOR)),
            Text(str(task['priority']) if task['priority'] else '-', style=Style(color=color if marked_task_id == task['task_id'] else constants.TABLE_TEXT_COLOR)),
            Text(day_offset_message, style=Style(color=color if marked_task_id == task['task_id'] else constants.TABLE_TEXT_COLOR))
        )
    
    general_utils.cache_task_ids(cfg=cfg, cache=task_ids_cache)
        
    return table
    

    
    

    



import json
from rich import print

import girok.utils.general as general_utils
import girok.api.auth as auth_api

def match_passwords(pwd, confirm_pwd):
    if pwd != confirm_pwd:
        print("Confirm password does not match.")
        exit(0)
        

def remove_access_token(config_path):
    with open(config_path, 'r') as f:
        org_data = json.load(f)
        del org_data['access_token']
        del org_data['email']
        
    general_utils.write_json(config_path, org_data)
        
        
def get_access_token_from_json(fpath):
    with open(fpath, 'r') as f:
        data = json.load(f)
        if "access_token" in data:
            return data['access_token']
        else:
            return None
        

def get_user_email_from_json(fpath):
    with open(fpath, 'r') as f:
        data = json.load(f)
        if "email" in data:
            return data['email']
        else:
            return None
        

def build_jwt_header(fpath):
    return {
        "Authorization": "Bearer " + get_access_token_from_json(fpath)
    }
        

def is_logged_in(access_token):
    resp = auth_api.validate_access_token(access_token)
    return True if resp.status_code == 200 else False
        

def store_access_token_to_json(fpath, access_token):
    data = {"access_token": access_token}
    general_utils.update_json(fpath, data)
    


import typer
import girok.calendar_cli.calendar_main as calendar_main

app = typer.Typer(rich_markup_mode='rich')

@app.command("cal")
def show_calendar():
    app = calendar_main.Entry()
    app.run()

